require 'rspec'
require'minilex'

describe "A simple lexer" do
  let(:lexer) do
    Minilex::Lexer.new do
      skip :whitespace, /\s+/
      tok :digit, /\d/
    end
  end

  it "raises an error on unrecognized input" do
    expect do
      lexer.lex('123abc')
    end.to raise_error(Minilex::UnrecognizedInput)
  end

  it "returns a single :eos token on empty input" do
    lexer.lex('').should == [[:eos]]
  end

  it "recognizes a single digit" do
    lexer.lex('1').should == [[:digit, '1', 1, 0], [:eos]]
  end

  it "recognizes multiple digits" do
    lexer.lex('123').should ==
      [[:digit, '1', 1, 0], [:digit, '2', 1, 1], [:digit, '3', 1, 2], [:eos]]
  end

  it "skips whitespace" do
    lexer.lex('1 2 3').should ==
      [[:digit, '1', 1, 0], [:digit, '2', 1, 2], [:digit, '3', 1, 4], [:eos]]
  end

  it "keeps track of line numbers" do
    lexer.lex("1\n2\n3").should ==
      [[:digit, '1', 1, 0], [:digit, '2', 2, 0], [:digit, '3', 3, 0], [:eos]]
  end

  it "keeps track of line offsets" do
    lexer.lex(" 1\n  2\n   3").should ==
      [[:digit, '1', 1, 1], [:digit, '2', 2, 2], [:digit, '3', 3, 3], [:eos]]
  end
end

describe "A simple lexer with converter" do
  let(:lexer) do
    Minilex::Lexer.new do
      skip :whitespace, /\s+/
      tok :digit, /\d/, :integer

      def integer(str); Integer(str); end
    end
  end

  it "convert the digit value to an integer" do
    lexer.lex('123').should ==
      [[:digit, 1, 1, 0], [:digit, 2, 1, 1], [:digit, 3, 1, 2], [:eos]]
  end
end

describe "Overriding how tokens are made" do
  let(:lexer) do
    Minilex::Lexer.new do
      skip :whitespace, /\s+/
      tok :digit, /\d/

      def append_token(id, value); tokens << Integer(value) ** 2; end
      def append_eos; tokens << "Zanzibar!"; end
    end
  end

  it "returns tokens from the overwritten :append_{token|eos} methods" do
    lexer.lex('123').should == [1, 4, 9, "Zanzibar!"]
  end
end

